#include "ros/node.h"

#include "visualization_msgs/Marker.h"
#include "visualization_msgs/MarkerArray.h"

#include <tf/transform_broadcaster.h>
#include <tf/tf.h>

int main( int argc, char** argv )
{
  ros::init( argc, argv );

  ros::Node* node = new ros::Node( "marker_test", ros::Node::DONT_HANDLE_SIGINT );

  while ( !node->ok() )
  {
    usleep( 10000 );
  }

  node->advertise<visualization_msgs::Marker>( "visualization_marker", 0 );
  node->advertise<visualization_msgs::MarkerArray>( "visualization_marker_array", 0 );

  tf::TransformBroadcaster tf_broadcaster;

  usleep( 1000000 );

  ros::Time tm(ros::Time::now());
  tf::Transform t;
  t.setIdentity();
  //  tf_broadcaster.sendTransform(tf::Stamped<tf::Transform>(t, tm, "base", "map"));

#if 0
  for ( int i = -50; i < 50; ++i )
  {
    visualization_msgs::Marker marker;
    marker.header.frame_id = "base_link";
    marker.header.stamp = ros::Time();
    marker.ns = "marker_test";
    marker.id = i;
    marker.type = visualization_msgs::Marker::ARROW;
    marker.action = visualization_msgs::Marker::ADD;
    marker.pose.position.x = 1;
    marker.pose.position.y = (i*2);
    marker.pose.position.z = 2;
    marker.pose.orientation.x = 0.0;
    marker.pose.orientation.y = 0.0;
    marker.pose.orientation.z = 0.0;
    marker.pose.orientation.w = 1.0;
    marker.scale.x = 0.2;
    marker.scale.y = 0.2;
    marker.scale.z = 0.2;
    marker.color.r = 0.0;
    marker.color.g = 1.0;
    marker.color.b = 0.0;
    marker.color.a = 1.0;
    //marker.lifetime = ros::Duration(10.0);
    node->publish( "visualization_marker", marker );
  }
#elif 0
  visualization_msgs::MarkerArray array;
  int i = 0;
  for (int x = 0; x < 20; ++x)
  {
    for (int y = 0; y < 10; ++y)
    {
      for (int z = 0; z < 10; ++z)
      {
        visualization_msgs::Marker marker;
        marker.header.frame_id = "base_link";
        marker.header.stamp = ros::Time();
        marker.ns = "marker_test";
        marker.id = i++;
        marker.type = visualization_msgs::Marker::CUBE;
        marker.action = visualization_msgs::Marker::ADD;
        marker.pose.position.x = x * 0.1f;
        marker.pose.position.y = y * 0.1f;
        marker.pose.position.z = z * 0.1f;
        marker.pose.orientation.x = 0.0;
        marker.pose.orientation.y = 0.0;
        marker.pose.orientation.z = 0.0;
        marker.pose.orientation.w = 1.0;
        marker.scale.x = 0.05;
        marker.scale.y = 0.05;
        marker.scale.z = 0.05;
        marker.color.r = 0.0;
        marker.color.g = 0.0;
        marker.color.b = 1.0;
        marker.color.a = 1.0;
        array.markers.push_back(marker);
      }
    }
  }
  node->publish( "visualization_marker_array", array );
#elif 0
  visualization_msgs::Marker marker;
  marker.header.frame_id = "base_link";
  marker.header.stamp = ros::Time();
  marker.ns = "marker_test";
  marker.id = 0;
  marker.type = visualization_msgs::Marker::CUBE_LIST;
  marker.action = visualization_msgs::Marker::ADD;
  marker.pose.orientation.x = 0.0;
  marker.pose.orientation.y = 0.0;
  marker.pose.orientation.z = 0.0;
  marker.pose.orientation.w = 1.0;
  marker.scale.x = 0.05;
  marker.scale.y = 0.05;
  marker.scale.z = 0.05;
  marker.color.r = 0.0;
  marker.color.g = 0.0;
  marker.color.b = 1.0;
  marker.color.a = 1.0;
  for (int x = 0; x < 100; ++x)
  {
    for (int y = 0; y < 100; ++y)
    {
      for (int z = 0; z < 10; ++z)
      {
        robot_msgs::Point p;
        p.x = x * 0.1f;
        p.y = y * 0.1f;
        p.z = z * 0.1f;

        marker.points.push_back(p);
      }
    }
  }
  node->publish( "visualization_marker", marker );
#elif 0
  visualization_msgs::Marker marker;
  marker.header.frame_id = "base_link";
  marker.header.stamp = ros::Time();
  marker.ns = "marker_test";
  marker.id = 0;
  marker.type = visualization_msgs::Marker::ARROW;
  marker.action = visualization_msgs::Marker::ADD;
  marker.pose.orientation.x = 0.0;
  marker.pose.orientation.y = 0.0;
  marker.pose.orientation.z = 0.0;
  marker.pose.orientation.w = 1.0;
  marker.scale.x = 0.02;
  marker.scale.y = 0.1;
  marker.color.r = 0.0;
  marker.color.g = 1.0;
  marker.color.b = 0.0;
  marker.color.a = 1.0;
  marker.points.resize(2);
  marker.points[0].x = 0.0f;
  marker.points[0].y = 0.0f;
  marker.points[0].z = 0.0f;
  marker.points[1].x = 1.0f;
  marker.points[1].y = 0.0f;
  marker.points[1].z = 0.0f;
  node->publish( "visualization_marker", marker );
#elif 1
  visualization_msgs::Marker marker;
  marker.header.frame_id = "base_link";
  marker.header.stamp = ros::Time();
  marker.ns = "marker_test";
  marker.id = 0;
  marker.type = visualization_msgs::Marker::POINTS;
  marker.action = visualization_msgs::Marker::ADD;
  marker.pose.orientation.x = 0.0;
  marker.pose.orientation.y = 0.0;
  marker.pose.orientation.z = 0.0;
  marker.pose.orientation.w = 1.0;
  marker.scale.x = 0.05;
  marker.scale.y = 0.05;
  marker.scale.z = 0.05;
  marker.color.r = 0.0;
  marker.color.g = 0.0;
  marker.color.b = 1.0;
  marker.color.a = 1.0;
  for (int x = 0; x < 1; ++x)
  {
    for (int y = 0; y < 1; ++y)
    {
      for (int z = 0; z < 1; ++z)
      {
        robot_msgs::Point p;
        p.x = x * 0.1f;
        p.y = y * 0.1f;
        p.z = z * 0.1f;

        marker.points.push_back(p);
      }
    }
  }
  node->publish( "visualization_marker", marker );
#elif 0
  int count = 40000;
  visualization_msgs::Marker marker;
  marker.header.frame_id = "base_link";
  marker.header.stamp = ros::Time();
  marker.ns = "marker_test";
  marker.id = 0;
  marker.type = visualization_msgs::Marker::POINTS;
  marker.action = visualization_msgs::Marker::ADD;
  marker.pose.position.x = 0.0;
  marker.pose.position.y = 0.0;
  marker.pose.position.z = 0.0;
  marker.pose.orientation.x = 0.0;
  marker.pose.orientation.y = 0.0;
  marker.pose.orientation.z = 0.0;
  marker.pose.orientation.w = 1.0;
  marker.scale.x = 0.5;
  marker.scale.y = 1.0;
  marker.color.r = 1.0;
  marker.color.a = 0.5;
  for ( int i = 0; i < count; ++i )
  {
    robot_msgs::Point p1, p2;
    p1.x = -1;
    p1.y = (i - count/2);
    p1.z = 0;
    marker.points.push_back(p1);
  }
  node->publish( "visualization_marker", marker );
#else

  int count = 40000;
  visualization_msgs::Marker marker;
  marker.header.frame_id = "base_link";
  marker.header.stamp = ros::Time();
  marker.ns = "marker_test";
  marker.id = 0;
  marker.type = visualization_msgs::Marker::LINE_LIST;
  marker.action = visualization_msgs::Marker::ADD;
  marker.pose.position.x = 0.0;
  marker.pose.position.y = 0.0;
  marker.pose.position.z = 0.0;
  marker.pose.orientation.x = 0.0;
  marker.pose.orientation.y = 0.0;
  marker.pose.orientation.z = 0.0;
  marker.pose.orientation.w = 1.0;
  marker.scale.x = 0.1;
  marker.color.r = 1.0;
  marker.color.a = 1.0;
  for ( int i = 0; i < count; ++i )
  {
    robot_msgs::Point p1, p2;
    p1.x = -1;
    p1.y = (i - count/2);
    p1.z = 0;
    p2.x = -1;
    p2.y = (i - count/2);
    p2.z = 1;
    marker.points.push_back(p1);
    marker.points.push_back(p2);
  }
  node->publish( "visualization_marker", marker );

  visualization_msgs::Marker line_marker;
  line_marker.header.frame_id = "base_link";
  line_marker.header.stamp = ros::Time();
  line_marker.ns = "marker_test";
  line_marker.id = count + 1;
  line_marker.type = visualization_msgs::Marker::LINE_STRIP;
  line_marker.action = visualization_msgs::Marker::ADD;
  line_marker.pose.position.x = 0.0;
  line_marker.pose.position.y = 0.0;
  line_marker.pose.position.z = 0.0;
  line_marker.pose.orientation.x = 0.0;
  line_marker.pose.orientation.y = 0.0;
  line_marker.pose.orientation.z = 0.0;
  line_marker.pose.orientation.w = 1.0;
  line_marker.scale.x = 0.1;
  line_marker.color.g = 1.0;
  line_marker.color.a = 1.0;
  for ( int i = -50; i < 50; ++i )
  {
    robot_msgs::Point p;
    p.x = 1;
    p.y = (i*2);
    p.z = 0;
    line_marker.points.push_back(p);
  }

  node->publish( "visualization_marker", line_marker );
#endif

  usleep( 1000000 );

  node->shutdown();
  delete node;


}
